#!/bin/bash
#
# Extract texture rasters.
# This script is updated for gdal 2.4+ could not work properly with older versions.
# 

FILTER=
F=mean
W=3
RES=0.0004

#RGB2IDX=~frankie/bin/rgb2idx.py
D=/ocean/data/ESA-SEOM/FromGamma/Sentinel1/delivery
T=/ocean/processings/processing_S-1_SEOM

#set -e

if [ -z "$(which pkinfo)" ]; then
	echo "Missing pktools"
	exit 1
fi

sites=$(cat <<EOF
Bierbrza
ElmCreek
Hobe
LittleWashita
Remedhus
TxSON
Yanco
EOF
)

for site in $(echo $sites)
do
	for texture in clay sand silt
	do
	gdalwarp -overwrite -of ENVI -te $(pkinfo -i $D/$site.*/reference/$site.dem -te|cut -d\  -f2-) -tr $(pkinfo -i $SM_TOOLS_TEXTURE/clay_nodata.tif -dx -dy|cut -d\  -f2,4) $SM_TOOLS_TEXTURE/clay_nodata.tif $T/$site/TEXTURE/clay_1deg
	gdalwarp -overwrite -of ENVI -te $(pkinfo -i $D/$site.*/reference/$site.dem -te|cut -d\  -f2-) -tr $(pkinfo -i $SM_TOOLS_TEXTURE/sand_nodata.tif -dx -dy|cut -d\  -f2,4) $SM_TOOLS_TEXTURE/sand_nodata.tif $T/$site/TEXTURE/sand_1deg
	gdalwarp -overwrite -of ENVI -te $(pkinfo -i $D/$site.*/reference/$site.dem -te|cut -d\  -f2-) -tr $(pkinfo -i $SM_TOOLS_TEXTURE/silt_nodata.tif -dx -dy|cut -d\  -f2,4) $SM_TOOLS_TEXTURE/silt_nodata.tif $T/$site/TEXTURE/silt_1deg
	clay_mean=$(gdalinfo -stats $T/$site/TEXTURE/clay_1deg|grep MEAN|cut -d= -f2)
	sand_mean=$(gdalinfo -stats $T/$site/TEXTURE/sand_1deg|grep MEAN|cut -d= -f2)
	silt_mean=$(gdalinfo -stats $T/$site/TEXTURE/silt_1deg|grep MEAN|cut -d= -f2)
	gdalwarp -overwrite -dstnodata $clay_mean $SM_TOOLS_TEXTURE/clay_nodata.tif /tmp/$$.clay.tif
	gdalwarp -overwrite -dstnodata $sand_mean $SM_TOOLS_TEXTURE/sand_nodata.tif /tmp/$$.sand.tif
	gdalwarp -overwrite -dstnodata $silt_mean $SM_TOOLS_TEXTURE/silt_nodata.tif /tmp/$$.silt.tif
	gdalwarp -overwrite -srcnodata None -r bilinear -of ENVI -tr $RES $RES -te $(pkinfo -i $D/$site.*/reference/$site.dem -te|cut -d\  -f2-) /tmp/$$.clay.tif  $T/$site/TEXTURE/clay
	gdalwarp -overwrite -srcnodata None -r bilinear -of ENVI -tr $RES $RES -te $(pkinfo -i $D/$site.*/reference/$site.dem -te|cut -d\  -f2-) /tmp/$$.sand.tif  $T/$site/TEXTURE/sand
	gdalwarp -overwrite -srcnodata None -r bilinear -of ENVI -tr $RES $RES -te $(pkinfo -i $D/$site.*/reference/$site.dem -te|cut -d\  -f2-) /tmp/$$.silt.tif  $T/$site/TEXTURE/silt
	rm -f /tmp/$$.clay.tif /tmp/$$.sand.tif /tmp/$$.silt.tif
	done

#	pushd $T/$site/TEXTURE
#
#	CLAY=clay
#	SAND=sand
#	SILT=silt
#	if [ ! -z $FILTER ]; then
#		CLAY=clay_$F$W
#		SAND=sand_$F$W
#		SILT=silt_$F$W
#		pkfilter -dx $W -dy $W -i clay -o $CLAY -f $F
#		pkfilter -dx $W -dy $W -i sand -o $SAND -f $F
#		pkfilter -dx $W -dy $W -i silt -o $SILT -f $F
#	fi
#	echo "Compositing ..." 
#	gdal_translate -scale 0 1 -of "ENVI" -ot Byte clay clay_gtc_scaled
#	gdal_translate -scale 0 1 -of "ENVI" -ot Byte sand sand_gtc_scaled
#	gdal_translate -scale 0 1 -of "ENVI" -ot Byte silt silt_gtc_scaled
#	rm -f texture.tif && gdal_merge.py -o texture.tif -separate clay_gtc_scaled sand_gtc_scaled silt_gtc_scaled
#	$RGB2IDX texture.tif texture_ct.tif
#	echo "STCODE CLAY% SAND%" >texture_smosar.txt
#	CLAY_MEAN=$(pkinfo -nodata 0 -stats -i clay|cut -d' ' -f6|awk '{printf "%.2f\n",$0*100}')
#	SAND_MEAN=$(pkinfo -nodata 0 -stats -i sand|cut -d' ' -f6|awk '{printf "%.2f\n",$0*100}')
#	echo "0 $CLAY_MEAN $SAND_MEAN" >>texture_smosar.txt
#	gdalinfo texture_ct.tif|grep -E '^[ 0-9]+: '|sed -e 's/: /,/'|awk -F, '{printf "%d %.2f %.2f\n",$1,100*$2/256,100*$3/255}'|grep -E '^1 ' >>texture_smosar.txt
#	gdalinfo texture_ct.tif|grep -E '^[ 0-9]+: '|sed -e 's/: /,/'|grep -v '0,0,0,255'| awk -F, '{printf "%d %.2f %.2f\n",$1,100*$2/256,100*$3/255}' |grep -E -v '^1 ' >>texture_smosar.txt
#	gdal_translate -of ENVI texture_ct.tif texture_smosar
#
#	popd
done


