#!/bin/sh -e

d=0.004

if [ $# -lt 2 ]
then
	echo "usage: $0 file size"
	exit
fi

$SM_TOOLS/etc/grass_check.sh || exit 1

d=$2
D=$1

cd $R 

if [ -z "$USE_FIELDS" ]
then
	echo "Name|S-1 Date|S-1 Time|Calibrated SM|Max Depth(m)|Station Date|Station Time|Long|Lat|SMOAR SM*100|Mean ${d}deg|StdDev ${d}deg|Min ${d}deg|Max ${d}deg|Cells ${d}deg|NULL cells ${d}deg"
	for line in $(cat $D)
	do
		file=$(echo $line|cut -d\| -f1)
		f=$(basename $file)
		dir=$(dirname $(dirname $file))
		file2="$dir/$f"
		x=$(echo $line|cut -d\| -f9)
		y=$(echo $line|cut -d\| -f10)
		rest=$(echo $line|cut -d\| -f2-)
		ulx=$(perl -E "say $x-$d")
		uly=$(perl -E "say $y+$d")
		lrx=$(perl -E "say $x+$d")
		lry=$(perl -E "say $y-$d")
		[ $($G g.list t=rast pat="$f"|wc -l) -eq 1 ] || $G r.external --o --quiet input=$file output=$f || $G r.external --o --quiet input=$file2 output=$f
		$G g.region --quiet  rast=$f
		val=$($G r.what null=0 map=$f coordinates=$x,$y|cut -d\| -f4)
		$G g.region --quiet  rast=$f n=$uly s=$lry w=$ulx e=$lrx 
		unset mean stddev min max cells null_cells
		eval $($G r.univar map=$f -g)
		echo "$rest|$val|$mean|$stddev|$min|$max|$cells|$null_cells"
	done

else

	echo "Name|S-1 Date|S-1 Time|Calibrated SM|Max Depth(m)|Station Date|Station Time|Long|Lat|SMOAR SM*100|Mean Field|StdDev Field|Min Field|Max Field|Cells Field|NULL cells Field"
	for line in $(cat $D)
	do
		file=$(echo $line|cut -d\| -f1)
		f=$(basename $file)
		d=$(dirname $(dirname $file))
		file2="$d/$f"
		x=$(echo $line|cut -d\| -f9)
		y=$(echo $line|cut -d\| -f10)
		rest=$(echo $line|cut -d\| -f2-)
		[ $($G g.list t=rast pat="$f"|wc -l) -eq 1 ] || $G r.external --o --quiet input=$file output=$f || $G r.external --o --quiet input=$file2 output=$f
		$G g.region --quiet  rast=$f
		val=$($G r.what null=0 map=$f coordinates=$x,$y|cut -d\| -f4)
		polygon=$(spatialite -silent $SM_TOOLS_DB "select ST_AsText(polygon) from Sensor where ST_X(location)=$x and ST_Y(location)=$y")
		centroid=$(spatialite -silent $SM_TOOLS_DB "select ST_AsText(ST_Centroid(polygon)) from Sensor where ST_X(location)=$x and ST_Y(location)=$y")
        np=$(echo $polygon|sed -e 's/POLYGON/ /' -e 's/((//' -e 's/))//' -e 's/,/\n/g'|wc -l)
		c=$(echo $centroid|sed -e 's/POINT(//' -e 's/)//')
		temp=$(mktemp -q)
        cat >$temp <<EOF
B $np
$(echo $polygon|sed -e 's/POLYGON/ /' -e 's/((//' -e 's/))//' -e 's/,/\n/g' )
C 1 1
 $c
 1 1
EOF
        $G v.in.ascii -n input=$temp output=field format=standard --q --o
        rm -f $temp
		$G g.region vect=field
		$G v.to.rast use=val value=1 in=field out=MASK --o
		unset mean stddev min max cells null_cells
		eval $($G r.univar map=$f -g)
		echo "$rest|$val|$mean|$stddev|$min|$max|$cells|$null_cells"
		$G r.mask -r
	done

fi
