#!/bin/sh

if [ $# -ne 3 ]
then
	echo "Usage: $0 <image list> <track_label> <outprefix>"
	exit 2
fi

L=$1
TRACK=$2
O=$3

if [ $($G g.list pat="smap_grid_*km_${TRACK}" t=vect|wc -l) -eq 0 ]
then
	echo "SMAP grid at required resolutions not found"
	exit 3
fi

set -e

# statistics are simplified by pre-rasterizing for bigger resolutions
# it needs a weighted by area approach when raster and grid are almost
# of the same order, i.e. very few pixels are involved.
REPORT_1=$(mktemp -q)
REPORT_2=$(mktemp -q)

#for RES in 36 25 12d5 9 3 1
#for RES in 36 25 12d5 9
for RES in 1
do
	V="smap_grid_${RES}km_${TRACK}"
	echo "date|cat|count|tot|perc|mean|stddev|type|reference|refence date|type" >${O}_${V}_report.txt
done

PREFIX=$(spatialite $SM_TOOLS_DB "select uri from Path where id=1")
for line in $(grep -E -v '#^' $L)
do
	R=$PREFIX/$(echo $line|cut -d\| -f1)
	D=$(echo $line|cut -d\| -f2)
	T=$(echo $line|cut -d\| -f3)
	B=$(basename $R)
	$G r.external in=$R out=$B --o
	#for RES in 36 25 12d5 9 3 1
	#for RES in 36 25 12d5 9
	for RES in 1
	do
		V="smap_grid_${RES}km_${TRACK}"
		R_RES=$(pkinfo -dx -dy -i $R|cut -d' ' -f4,2 --output-delimiter=,)
		EW_RES=$(echo $R_RES|cut -d, -f1)
		NS_RES=$(echo $R_RES|cut -d, -f2)
		$G g.region -a rast=$B ewres=$EW_RES nsres=$NS_RES
		$G r.mask -r || true
		$G v.to.rast in=$V use=cat out=$V --o
		eval $($G r.info map=$V -g|grep ncats)
		if [ $ncats -gt 0 ]
		then
			$G i.segment.stats -c map=$V rasters=$V raster_statistics= area_measures=area csvfile=$REPORT_1 --o
			$G r.mask raster=$B --o
			$G i.segment.stats -c map=$V rasters=$B raster_statistics=mean,stddev area_measures=area csvfile=$REPORT_2 --o
			tail -n +2 $REPORT_1 | sort -n -t \| -k 1 >${O}_${V}_report1.txt
			tail -n +2 $REPORT_2 | sort -n -t \| -k 1 >${O}_${V}_report2.txt
			for l in $(cat ${O}_${V}_report2.txt)
			do
				k=$(echo $l|cut -d\| -f1)
				a=$(grep -E "^$k\|" ${O}_${V}_report1.txt|cut -d\| -f2)
				val=$($SM_TOOLS/misc/get_smapsmosascat_value.sh $D $RES $TRACK $k)
				echo "$l|$a|$val|$D|$T" |awk -F\| \
					'{printf("%s|%d|%d|%d|%d|%f|%f|%s|%f|%s|%s\n",$9,$1,$2,$5,100*($2/$5),$3,$4,$6,$7,$8,$10)}' >>${O}_${V}_report.txt
			done
			rm -f ${O}_${V}_report?.txt
		fi
	done
	rm -f $REPORT_1 $REPORT_2 
done

$G_CLEAN

exit 0
