#!/bin/sh
# This simple script combines two bands (mean and stddev) and produces a TIF with the same resolution and with bands:
# band1: mean
# band2: stddev
# band3: CoeffVar=stddev/mean*10000
# outputs them as different rasters with the same geometry. It combines output
# in a hybrid resolution, starting from a low-res image and an high-res one.
# 

if [ $# -ne 2 ]
then
	echo "usage: $0 <file list> <outdir>"
	exit 1
fi

$SM_TOOLS/etc/grass_check.sh || exit 1

set -e

LIST=$1
OUTDIR=$2

for line in $(cat $LIST|grep -E -v '^#')
do
	# line is: img_mean,img_stddev
	mean=$(echo $line|cut -d, -f1)
	stddev=$(echo $line|cut -d, -f2)
	O=$(basename $mean .img_mean)

	$G r.external in=$mean out=mean --o
	$G r.external in=$stddev out=stddev --o

	$G g.region -a rast=mean
	$G r.mask -r || true
	$G r.mapcalc expression="cofvar=10000.0*stddev/abs(mean)" --o
	for r in mean stddev cofvar
	do
		$G r.null null=0 map=$r 
	done

    	$G r.colors map=mean rules=$SM_TOOLS/etc/colors.txt
    	$G g.region rast=mean
	$G r.out.png -w -t in=mean out=${OUTDIR}/${O}_mean.png compress=9 --o

    	$G r.colors map=stddev rules=$SM_TOOLS/etc/colors_stddev.txt
    	$G g.region rast=stddev
	$G r.out.png -w -t in=stddev out=${OUTDIR}/${O}_stddev.png compress=9 --o

	$G i.group in=mean,stddev,cofvar group=all --o
	$G r.out.gdal -c in=all out=${OUTDIR}/${O}_combined.tif nodata=0 --o 

	$G g.remove t=group name=all -f
done

$G_CLEAN

exit 0
