#!/bin/bash

set -e

if [ $# -ne 2 ]
then
	echo "usage: $0 date smap|smos|ascat|smap1|smap3"
fi

date=$1
sat=$2

case $sat in
	smap1)
		SAT='SMAP1'
		RES='1km'
		GAIN=1
		;;
	smap3)
		SAT='SMAP3'
		RES='3km'
		GAIN=1
		;;
	smap|smap36)
		SAT='SMAP36'
		RES='36km'
		GAIN=1
		;;
	smos)
		SAT='SMOS'
		RES='25km'
		if [ $(echo $date|cut -d- -f1) = '2017' ]
		then
			GAIN=0.0003051850
		else
			GAIN=1
		fi
		;;
	ascat)
		SAT='ASCAT'
		RES='12d5km'
		GAIN=0.010
		;;
	*)
		echo 'Wrong type of map selected, use smap|smap1|smap3|smos|ascat'
		exit 2
		;;
esac

isodate=$(echo $date|sed -e's/-//g')
mosaic="mosaic_${SAT}_Medi_$isodate"
LIST=$(grep -E "^[^#]" <<EOF
#/net/milos/ocean/SMOSAR/SMOSAR_SEOM/Output_FD_mv_maps/Medi.lists/list_th_mean_Medi.D008.txt
#/net/milos/ocean/SMOSAR/SMOSAR_SEOM/Output_FD_mv_maps/Medi.lists/list_th_mean_Medi.D037.txt
#/net/milos/ocean/SMOSAR/SMOSAR_SEOM/Output_FD_mv_maps/Medi.lists/list_th_mean_Medi.D066.txt
#/net/milos/ocean/SMOSAR/SMOSAR_SEOM/Output_FD_mv_maps/Medi.lists/list_th_mean_Medi.D110.txt
#/net/milos/ocean/SMOSAR/SMOSAR_SEOM/Output_FD_mv_maps/Medi.lists/list_th_mean_Medi.D139.txt
#/net/milos/ocean/SMOSAR/SMOSAR_SEOM/Output_FD_mv_maps/Medi.lists/list_th_mean_Medi.D168.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D007_MN03_RL3320/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D007_MN05_RL4150/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D008_MN02_RL3478/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D008_MN07_RL4480/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D021_MN06_RL3850/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D022_MN01_RL3820/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D022_MN03_RL3400/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D022_MN04_RL4550/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D036_MN02_RL3180/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D036_MN05_RL4000/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D037_MN03_RL3720/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D037_MN03_RL4550/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D051_MN03_RL3300/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D051_MN06_RL4400/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D052_MN05_RL4350/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D065_MN03_RL3220/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D065_MN03_RL3850/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D066_MN03_RL3720/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D066_MN03_RL4650/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D080_MN03_RL3320/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D080_MN04_RL4150/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D081_MN07_RL4350/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D094_MN03_RL3220/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D094_MN03_RL3850/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D095_MN04_RL3720/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D095_MN05_RL4650/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D109_MN02_RL3279/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D109_MN05_RL4000/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D110_MN03_RL3650/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D110_MN04_RL4480/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D124_MN02_RL3300/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D124_MN06_RL4400/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D125_MN05_RL4350/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D138_MN02_RL3180/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D138_MN03_RL3850/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D139_MN03_RL3720/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D139_MN03_RL4550/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D153_MN03_RL3200/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D153_MN03_RL4300/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D154_MN01_RL3475/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D154_MN06_RL4350/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D167_MN03_RL3220/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D167_MN03_RL3850/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D168_MN03_RL3720/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
/net/milos/ocean/SMOSAR/SMOSAR_Area_Med/tools/Med_S1_D168_MN06_RL4650/list_FD_run_n4_v7_3_20181120_th33_mean_sel_mosaic.txt
EOF
)
list=$(for i in $(seq 0 5); do date --date="$date + $i day" +'%Y-%m-%d'; done|sed -e 's/$/,/'|perl -00pe 's/,(?!.*,)//s')
regexp=$(echo $list|sed -e 's/, /|/g')
TEMPDIR=$(mktemp -dq)
for i in $(cat $LIST|grep -E "$regexp"|cut -d, -f1)
do
	TRACK=$(dirname $i|sed -e 's/Med_S1_/Medi./'|perl -pe 's/.+(Medi\.D[0-9]{3}[A-Z0-9_]*).+/$1/'|sed -e 's/\./_/')
	ISODATE=$(basename $i|perl -pe 's/.+([0-9]{8}).+/$1/')
	FILE=$SM_TOOLS_OUTPUTDIR/smap_grids_sampling/${SAT}_${ISODATE}_$TRACK.txt
	if [ -f $FILE ]
	then
		F=$(basename $FILE .txt)
		echo "${sat},val" >$TEMPDIR/input.csv
		echo "Integer,Real" >$TEMPDIR/input.csvt
		cat $FILE|grep -E -v '\|0$'|sed -e 's/|/,/'|awk -F, "{printf \"%d,%f\\n\",\$1,\$2*$GAIN*4500+500}" >>$TEMPDIR/input.csv
		#cat $FILE|grep -E -v '\|0$'|sed -e 's/|/,/'|awk -F, "{printf \"%d,%f\\n\",\$1,\$2*$GAIN}" >>$TEMPDIR/input.csv
		if [ $(cat $TEMPDIR/input.csv|wc -l) -gt 1 ]
		then
			$G_EASE2 db.in.ogr in=$TEMPDIR/input.csv out=${SAT}_$TRACK --o
			$G_EASE2 g.copy vect=smap_grid_${RES}_$TRACK,smap_grid_${RES}_${TRACK}_val --o
			$G_EASE2 v.db.join map=smap_grid_${RES}_${TRACK}_val column=cat other_table=${SAT}_$TRACK other_column=${sat} 
		fi
	fi
done

vlist=$($G_EASE2 g.list t=vect pat="smap_grid_${RES}_*_val" sep=,)
$G_EASE2 g.region vect=$vlist res=500
for i in $($G_EASE2 g.list t=vect pat="smap_grid_${RES}_*_val")
do
	$G_EASE2 v.to.rast in=$i out=$i use=attr attr='val' where="val!=''" --o
done

rlist=$($G_EASE2 g.list t=rast pat="smap_grid_${RES}_*_val" sep=,)
$G_EASE2 r.patch in=$rlist out=$mosaic --o
$G_EASE2 r.null setnull=0 map=$mosaic
$G_EASE2 r.colors map=$mosaic rules=$SM_TOOLS/etc/colors.txt
$G_EASE2 r.out.png -w -t in=$mosaic out=$SM_TOOLS_OUTPUTDIR/$mosaic.png --o

# do not cleanup to avoid removing intermediate data.
#$G_CLEAN

exit 0
