#!/bin/sh
#
# This scripts extracts zonal statistics for LAI maps in Apulian Tavoliere
# using base land cover maps for the area which is available in different months.
#


#	 SENSAGRI LAI green & LAI brown product. GPR retrieval algorithm version 0.30. Software version 0.20.
#	 Processing Date: 12-Nov-2018. Processing Time [s]: 173
#	 Band 1 = LAI Total [m2/m2] including uncertainty thresholds masks of LAI green (40) and LAI brown (40). Masked pixels are 'NaN'.
#	 Band 2 = LAI green [m2/m2].
#	 Band 3 = LAI green Standard Deviation (SD) [m2/m2].
#	 Band 4 = LAI green Coefficient of Variation (CV) used as uncertainty threshold [%].
#	 Band 5 = LAI brown [m2/m2].
#	 Band 6 = LAI brown Standard Deviation (SD) [m2/m2].
#	 Band 7 = LAI brown Coefficient of Variation (CV) used as uncertainty threshold [%].
#	 Band 8 = Classification band: 0=Out of uncertainty threshold, 1=LAI green, 2= LAI green&brown 3=LAI brown.
#	 Band 9 = S2 SCL (Sentinel-2 Scene classification)
#	 The S2 SCL cloud mask is applied.
# 
#

set -e

TAG=lai_10m_initial_release
LC=apuliantavoliere
OUTDIR=$SM_TOOLS/streams_crossing/results
LAI_BANDS=1
#LAI_BANDS=$(seq 9)
YEAR=2017

# The LAI maps are 20x20m^2 wide.
MINCELLS=25

# a pair of silly functions ...
getcat() {
	if [ $1 -ge 100000 ]; then
		echo $(expr $1 - 100000)
		return
	fi
	if [ $1 -ge 90000 ]; then
		echo $(expr $1 - 90000)
		return
	fi
	if [ $1 -ge 80000 ]; then
		echo $(expr $1 - 80000)
		return
	fi
	if [ $1 -ge 70000 ]; then
		echo $(expr $1 - 70000)
		return
	fi
	if [ $1 -ge 60000 ]; then
		echo $(expr $1 - 60000)
		return
	fi
	if [ $1 -ge 50000 ]; then
		echo $(expr $1 - 50000)
		return
	fi
	if [ $1 -ge 40000 ]; then
		echo $(expr $1 - 40000)
		return
	fi
	if [ $1 -ge 30000 ]; then
		echo $(expr $1 - 30000)
		return
	fi
}

getmap() {
	if [ $1 -ge 100000 ]; then
		echo ${LC}_10$YEAR
		return
	fi
	if [ $1 -ge 90000 ]; then
		echo ${LC}_09$YEAR
		return
	fi
	if [ $1 -ge 80000 ]; then
		echo ${LC}_08$YEAR
		return
	fi
	if [ $1 -ge 70000 ]; then
		echo ${LC}_07$YEAR
		return
	fi
	if [ $1 -ge 60000 ]; then
		echo ${LC}_06$YEAR
		return
	fi
	if [ $1 -ge 50000 ]; then
		echo ${LC}_05$YEAR
		return
	fi
	if [ $1 -ge 40000 ]; then
		echo ${LC}_04$YEAR
		return
	fi
	if [ $1 -ge 30000 ]; then
		echo ${LC}_03$YEAR
		return
	fi
}

ROWS=$(spatialite -batch -noheader $SM_TOOLS_DB <<EOF
select p.filename,date(p.reference_datetime) from Product as p, Site as s 
where p.tag='$TAG' and strftime('%Y',p.reference_datetime)='$YEAR' 
and cast(strftime('%m',reference_datetime) as integer) <= 5 
and s.id=1 and ST_Intersects(p.footprint,s.aoi)>0;
EOF
)

R_CNT=$(mktemp -q)
R_MIN=$(mktemp -q)
R_MAX=$(mktemp -q)
R_AVG=$(mktemp -q)
R_STD=$(mktemp -q)
R_SKW=$(mktemp -q)
R_KUR=$(mktemp -q)

for r in $ROWS
do
	f=$(echo $r|cut -d\| -f1)
	d=$(echo $r|cut -d\| -f2)
	for i in $LAI_BANDS
	do
		echo "LAI $d $i" 
		if [ -f $OUTDIR/LAI_${d}_${i}_${LC}_winter_stats.txt ] 
		then 
			continue
		fi

		LAI=$SM_TOOLS_REPO/$f
		$G r.in.gdal band=$i in=$LAI out=LAI --o
		$G r.null setnull=0 map=LAI
		$G g.region rast=LAI 

		for m in $(perl -e 'for $i (3..5) {printf "%02d\n",$i}')
		do
			$G v.to.rast in=${LC}_${m}$YEAR out=${LC}_${m}$YEAR use=cat label=crop_en mem=8000 --o
		done
		for m in $(perl -e 'for $i (3..5) {printf "%02d\n",$i}')
		do
			ofs=$(expr 10000 \* $m)
			$G r.mapcalc expr="${LC}_${m}${YEAR}_$ofs = ${LC}_${m}$YEAR + $ofs" --o
		done

		# consolidating multiple LCs, this is site specific :-/ do we need to save them?
		$G r.patch in="${LC}_05${YEAR}_50000,${LC}_04${YEAR}_40000,${LC}_03${YEAR}_30000" out=${LC}_winter --o

		for season in winter 
		do
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_CNT method=count out=LAI_count --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_MIN method=min out=LAI_min --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_MAX method=max out=LAI_max --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_AVG method=average out=LAI_mean --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_STD method=stddev2 out=LAI_stddev --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_SKW method=skewness2 out=LAI_skewness --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_KUR method=kurtosis2 out=LAI_kurtosis --o
			echo "map|cat|crop|count|min|max|avg|stddev|skew|kurt" >$OUTDIR/LAI_${d}_${i}_${LC}_${season}_stats.txt
			for l in $($G r.category map=${LC}_$season sep=\||cut -d\| -f1) 
			do
				count=$(grep -E "^$l =" $R_CNT|cut -d' ' -f4|cut -d. -f1)
				# skip when number of cells is too low
				if [ $count -ge $MINCELLS ]
				then
					cat=$(getcat $l)
					map=$(getmap $l)
					crop=$($G r.category map=$map cat=$cat sep=\||cut -d\| -f2)
					min=$(grep -E "^$l =" $R_MIN|cut -d' ' -f4|sed -e 's/-nan/NA/')
					max=$(grep -E "^$l =" $R_MAX|cut -d' ' -f4|sed -e 's/-nan/NA/')
					avg=$(grep -E "^$l =" $R_AVG|cut -d' ' -f4|sed -e 's/-nan/NA/')
					stddev=$(grep -E "^$l =" $R_STD|cut -d' ' -f4|sed -e 's/-nan/NA/')
					skew=$(grep -E "^$l =" $R_SKW|cut -d' ' -f4|sed -e 's/-nan/NA/')
					kurt=$(grep -E "^$l =" $R_KUR|cut -d' ' -f4|sed -e 's/-nan/NA/')
					echo "$map|$cat|$crop|$count|$min|$max|$avg|$stddev|$skew|$kurt" >>$OUTDIR/LAI_${d}_${i}_${LC}_${season}_stats.txt
				fi
			done
		done
	done
done

ROWS=$(spatialite -batch -noheader $SM_TOOLS_DB <<EOF
select p.filename,date(p.reference_datetime) from Product as p, Site as s 
where p.tag='$TAG' and strftime('%Y',p.reference_datetime)='$YEAR' and 
cast(strftime('%m',reference_datetime) as integer) > 5 
and s.id=1 and ST_Intersects(p.footprint,s.aoi)>0;
EOF
)

for r in $ROWS
do
	f=$(echo $r|cut -d\| -f1)
	d=$(echo $r|cut -d\| -f2)
	for i in 1 #$(seq 9)
	do
		echo "LAI $d $i" 
		if [ -f $OUTDIR/LAI_${d}_${i}_${LC}_spring_stats.txt ] 
		then 
			continue
		fi

		LAI=$SM_TOOLS_REPO/$f
		$G r.external band=$i in=$LAI out=LAI --o
		$G r.null setnull=0 map=LAI
		$G g.region rast=LAI 

		for m in $(perl -e 'for $i (6..10) {printf "%02d\n",$i}')
		do
			$G v.to.rast in=${LC}_${m}${YEAR} out=${LC}_${m}$YEAR use=cat label=crop_en mem=8000 --o
		done
		for m in $(perl -e 'for $i (6..10) {printf "%02d\n",$i}')
		do
			ofs=$(expr 10000 \* $m)
			$G r.mapcalc expr="${LC}_${m}${YEAR}_$ofs = ${LC}_${m}${YEAR} + $ofs" --o
		done

		# consolidating multiple LCs, this is site specific :-/ do we need to save them?
		$G r.patch in="${LC}_062017_60000,${LC}_072017_70000,${LC}_082017_80000,${LC}_092017_90000,${LC}_102017_100000" out=${LC}_spring --o

		for season in spring 
		do
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_CNT method=count out=LAI_count --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_MIN method=min out=LAI_min --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_MAX method=max out=LAI_max --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_AVG method=average out=LAI_mean --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_STD method=stddev2 out=LAI_stddev --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_SKW method=skewness2 out=LAI_skewness --o
			$G r.stats.zonal2 -r base=${LC}_$season cover=LAI rules=$R_KUR method=kurtosis2 out=LAI_kurtosis --o
			echo "map|cat|crop|count|min|max|avg|stddev|skew|kurt" >$OUTDIR/LAI_${d}_${i}_${LC}_${season}_stats.txt
			for l in $($G r.category map=${LC}_$season sep=\||cut -d\| -f1) 
			do
				count=$(grep -E "^$l =" $R_CNT|cut -d' ' -f4|cut -d. -f1)
				# skip when number of cells is too low
				if [ $count -ge $MINCELLS ]
				then
					cat=$(getcat $l)
					map=$(getmap $l)
					crop=$($G r.category map=$map cat=$cat sep=\||cut -d\| -f2)
					min=$(grep -E "^$l =" $R_MIN|cut -d' ' -f4|sed -e 's/-nan/NA/')
					max=$(grep -E "^$l =" $R_MAX|cut -d' ' -f4|sed -e 's/-nan/NA/')
					avg=$(grep -E "^$l =" $R_AVG|cut -d' ' -f4|sed -e 's/-nan/NA/')
					stddev=$(grep -E "^$l =" $R_STD|cut -d' ' -f4|sed -e 's/-nan/NA/')
					skew=$(grep -E "^$l =" $R_SKW|cut -d' ' -f4|sed -e 's/-nan/NA/')
					kurt=$(grep -E "^$l =" $R_KUR|cut -d' ' -f4|sed -e 's/-nan/NA/')
					echo "$map|$cat|$crop|$count|$min|$max|$avg|$stddev|$skew|$kurt" >>$OUTDIR/LAI_${d}_${i}_${LC}_${season}_stats.txt
				fi
			done
		done
	done
done

rm -f $R_CNT $R_MIN $R_MAX $R_AVG $R_STD $R_SKW $R_KUR

$G_CLEAN
