#!/bin/sh
#
# This scripts extracts zonal statistics for NDVI maps in Apulian Tavoliere
# using base land cover maps for the area which is available in different months.
#

set -e

TAG=tillage_40m_initial_release
LC=apuliantavoliere
OUTDIR=$SM_TOOLS/streams_crossing/results
MINCELLS=7
YEAR=2017
R_CNT=$(mktemp -q)
 
# a pair of silly functions ...
getcat() {
	if [ $1 -ge 100000 ]; then
		echo $(expr $1 - 100000)
		return
	fi
	if [ $1 -ge 90000 ]; then
		echo $(expr $1 - 90000)
		return
	fi
	if [ $1 -ge 80000 ]; then
		echo $(expr $1 - 80000)
		return
	fi
	if [ $1 -ge 70000 ]; then
		echo $(expr $1 - 70000)
		return
	fi
	if [ $1 -ge 60000 ]; then
		echo $(expr $1 - 60000)
		return
	fi
	if [ $1 -ge 50000 ]; then
		echo $(expr $1 - 50000)
		return
	fi
	if [ $1 -ge 40000 ]; then
		echo $(expr $1 - 40000)
		return
	fi
	if [ $1 -ge 30000 ]; then
		echo $(expr $1 - 30000)
		return
	fi
}

getmap() {
	if [ $1 -ge 100000 ]; then
		echo ${LC}_10$YEAR
		return
	fi
	if [ $1 -ge 90000 ]; then
		echo ${LC}_09$YEAR
		return
	fi
	if [ $1 -ge 80000 ]; then
		echo ${LC}_08$YEAR
		return
	fi
	if [ $1 -ge 70000 ]; then
		echo ${LC}_07$YEAR
		return
	fi
	if [ $1 -ge 60000 ]; then
		echo ${LC}_06$YEAR
		return
	fi
	if [ $1 -ge 50000 ]; then
		echo ${LC}_05$YEAR
		return
	fi
	if [ $1 -ge 40000 ]; then
		echo ${LC}_04$YEAR
		return
	fi
	if [ $1 -ge 30000 ]; then
		echo ${LC}_03$YEAR
		return
	fi
}

ROWS=$(spatialite -batch -noheader $SM_TOOLS_DB <<EOF
select p.filename,date(p.reference_datetime) from Product as p, Site as s 
where p.tag='$TAG' and strftime('%Y',p.reference_datetime)='$YEAR' 
and cast(strftime('%m',reference_datetime) as integer) <= 5 
and s.id=1 and ST_Intersects(p.footprint,s.aoi)>0;
EOF
)


for r in $ROWS
do
	f=$(echo $r|cut -d\| -f1)
	d=$(echo $r|cut -d\| -f2)
	echo "TLG $d" 
	if [ -f $OUTDIR/TLG_${d}_${LC}_winter_stats.txt ] 
	then 
		continue
	fi

	TLG=$SM_TOOLS_REPO/$f
	$G r.in.gdal -e in=$TLG out=TLG --o
	$G g.region rast=TLG 
	$G r.null -c map=TLG --o
	$G r.null setnull=255,128,129 map=TLG
	$G r.category map=TLG sep=: rules=- <<EOF
1:Harrowed
2:Ploughed
EOF

	# winter

	for m in $(perl -e 'for $i (3..5) {printf "%02d\n",$i}')
	do
		$G v.to.rast in=${LC}_${m}${YEAR} out=${LC}_${m}$YEAR use=cat label=crop_en mem=8000 --o
	done
	for m in $(perl -e 'for $i (3..5) {printf "%02d\n",$i}')
	do
		ofs=$(expr 10000 \* $m)
		$G r.mapcalc expr="${LC}_${m}${YEAR}_$ofs = ${LC}_${m}$YEAR + $ofs" --o
	done

	# consolidating multiple LCs, this is site specific :-/ do we need to save them?
	$G r.patch in="${LC}_052017_50000,${LC}_042017_40000,${LC}_032017_30000" out=${LC}_winter --o

	for season in winter 
	do
		$G r.stats.zonal2 -r base=${LC}_$season cover=TLG rules=$R_CNT method=count out=tlg_count --o
		$G r.statistics base=${LC}_$season cover=TLG method=mode out=tlg_mode --o
		$G r.statistics base=${LC}_$season cover=TLG method=diversity out=tlg_div --o
		echo "map|cat|count|crop|tillage|diversity" >$OUTDIR/TLG_${d}_${LC}_${season}_stats.txt
		for l in $($G r.category map=${LC}_$season sep=\||cut -d\| -f1) 
		do
			count=$(grep -E "^$l =" $R_CNT|cut -d' ' -f4|cut -d. -f1)
			# skip when number of cells is too low
			if [ $count -ge $MINCELLS ]
			then
				cat=$(getcat $l)
				map=$(getmap $l)
				crop=$($G r.category map=$map cat=$cat sep=\||cut -d\| -f2)
				mode=$($G r.category map=tlg_mode cat=$l sep=\||cut -d\| -f2)
				div=$($G r.category map=tlg_div cat=$l sep=\||cut -d\| -f2)
				tlg=$($G r.category map=TLG cat=$mode sep=\||cut -d\| -f2)
				echo "$map|$cat|$count|$crop|$tlg|$div" >>$OUTDIR/TLG_${d}_${LC}_${season}_stats.txt
			fi
		done
	done
done

# spring TLG

ROWS=$(spatialite -batch -noheader $SM_TOOLS_DB <<EOF
select p.filename,date(p.reference_datetime) from Product as p, Site as s 
where p.tag='$TAG' and strftime('%Y',p.reference_datetime)='$YEAR' 
and cast(strftime('%m',reference_datetime) as integer) > 5
and s.id=1 and ST_Intersects(p.footprint,s.aoi)>0;
EOF
)

for r in $ROWS
do
	f=$(echo $r|cut -d\| -f1)
	d=$(echo $r|cut -d\| -f2)
	echo "TLG $d" 
	if [ -f $OUTDIR/TLG_${d}_${LC}_spring_stats.txt ] 
	then 
		continue
	fi

	TLG=$SM_TOOLS_REPO/$f
	$G r.in.gdal -e in=$TLG out=TLG --o
	$G g.region rast=TLG 
	$G r.null -c map=TLG --o
	$G r.null setnull=255,128,129 map=TLG
	$G r.category map=TLG sep=: rules=- <<EOF
1:Harrowed
2:Ploughed
EOF

	# spring 

	for m in $(perl -e 'for $i (6..10) {printf "%02d\n",$i}')
	do
		$G v.to.rast in=${LC}_${m}${YEAR} out=${LC}_${m}$YEAR use=cat label=crop_en mem=8000 --o
	done
	for m in $(perl -e 'for $i (6..10) {printf "%02d\n",$i}')
	do
		ofs=$(expr 10000 \* $m)
		$G r.mapcalc expr="${LC}_${m}${YEAR}_$ofs = ${LC}_${m}${YEAR} + $ofs" --o
	done

	# consolidating multiple LCs, this is site specific :-/ do we need to save them?
	$G r.patch in="${LC}_062017_60000,${LC}_072017_70000,${LC}_082017_80000,${LC}_092017_90000,${LC}_102017_100000" out=${LC}_spring --o

	for season in spring 
	do
		$G r.stats.zonal2 -r base=${LC}_$season cover=TLG rules=$R_CNT method=count out=tlg_count --o
		$G r.statistics base=${LC}_$season cover=TLG method=mode out=tlg_mode --o
		$G r.statistics base=${LC}_$season cover=TLG method=diversity out=tlg_div --o
		echo "map|cat|count|crop|tillage|diversity" >$OUTDIR/TLG_${d}_${LC}_${season}_stats.txt
		for l in $($G r.category map=${LC}_$season sep=\||cut -d\| -f1) 
		do
			count=$(grep -E "^$l =" $R_CNT|cut -d' ' -f4|cut -d. -f1)
			# skip when number of cells is too low
			if [ $count -ge $MINCELLS ]
			then
				cat=$(getcat $l)
				map=$(getmap $l)
				crop=$($G r.category map=$map cat=$cat sep=\||cut -d\| -f2)
				mode=$($G r.category map=tlg_mode cat=$l sep=\||cut -d\| -f2)
				div=$($G r.category map=tlg_div cat=$l sep=\||cut -d\| -f2)
				tlg=$($G r.category map=TLG cat=$mode sep=\||cut -d\| -f2)
				echo "$map|$cat|$count|$crop|$tlg|$div" >>$OUTDIR/TLG_${d}_${LC}_${season}_stats.txt
			fi
		done
	done
done

rm -f $R_CNT

$G_CLEAN
