#!/bin/sh
#
# This scripts extracts zonal statistics for NDVI maps in Apulian Tavoliere
# using base land cover maps for the area which is available in different months.
#

set -e

TAG=ndvi_40m_initial_release
LC=spain
OUTDIR=$SM_TOOLS/streams_crossing/results
NDVI_BANDS=1
#NDVI_BANDS=$(seq 9)
YEAR=2017

# The NDVI maps are 40x40m^2 wide.
MINCELLS=7

ROWS=$(spatialite -batch -noheader $SM_TOOLS_DB <<EOF
select p.filename,date(p.reference_datetime) from Product as p, Site as s 
where p.tag='$TAG' and strftime('%Y',p.reference_datetime)='$YEAR' 
and s.id=5 and ST_Intersects(p.footprint,s.aoi)>0;
EOF
)

R_CAT=$(mktemp -q)
R_CNT=$(mktemp -q)
R_MIN=$(mktemp -q)
R_MAX=$(mktemp -q)
R_AVG=$(mktemp -q)
R_STD=$(mktemp -q)
R_SKW=$(mktemp -q)
R_KUR=$(mktemp -q)

for r in $ROWS
do
	f=$(echo $r|cut -d\| -f1)
	d=$(echo $r|cut -d\| -f2)

	for i in $NDVI_BANDS
	do
		echo "NDVI $d $i" 
		if [ -f $OUTDIR/NDVI_${d}_${i}_${LC}_stats.txt ]
		then
			continue
		fi

		NDVI=$SM_TOOLS_REPO/$f
		$G r.in.gdal band=$i in=$NDVI out=NDVI --o
		$G g.region rast=NDVI 
		$G r.null setnull=0 map=NDVI

		# This is a way to trim a looooot of polygons  
		$G r.clump in=${LC}_$YEAR out=${LC}_${YEAR}_clump --o
		$G r.area in=${LC}_${YEAR}_clump out=${LC}_${YEAR}_clump_1ha lesser=$MINCELLS --o
		$G r.clump in=${LC}_${YEAR}_clump_1ha out=${LC}_lc_clump --o
		$G r.mapcalc expr="${LC}_lc=${LC}_${YEAR}" --o

		# This is a trick to have a map cat -> land cover class
		#$G r.statistics base=${LC}_lc_clump cover=${LC}_lc method=mode out=LC_cat --o

		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=${LC}_lc rules=$R_CAT method=min out=NDVI_cat --o
		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=NDVI rules=$R_CNT method=count out=NDVI_count --o
		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=NDVI rules=$R_MIN method=min out=NDVI_min --o
		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=NDVI rules=$R_MAX method=max out=NDVI_max --o
		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=NDVI rules=$R_AVG method=average out=NDVI_mean --o
		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=NDVI rules=$R_STD method=stddev2 out=NDVI_stddev --o
		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=NDVI rules=$R_SKW method=skewness2 out=NDVI_skewness --o
		$G r.stats.zonal2 -r base=${LC}_lc_clump cover=NDVI rules=$R_KUR method=kurtosis2 out=NDVI_kurtosis --o
		echo "cat|crop|count|min|max|avg|stddev|skew|kurt" >$OUTDIR/NDVI_${d}_${i}_${LC}_stats.txt
		for cat in $(cut -d= -f1 $R_CAT) 
		do
			class=$(grep -E "^$cat =" $R_CAT|cut -d' ' -f4|cut -d. -f1)
			count=$(grep -E "^$cat =" $R_CNT|cut -d' ' -f4|cut -d. -f1)
			# skip when number of cells is too low
			if [ $count -ge $MINCELLS ]
			then
				crop=$($G r.category --v map=${LC}_lc cat=$class sep=\||cut -d\| -f2)
				min=$(grep -E "^$cat =" $R_MIN|cut -d' ' -f4|sed -e 's/-nan/NA/')
				max=$(grep -E "^$cat =" $R_MAX|cut -d' ' -f4|sed -e 's/-nan/NA/')
				avg=$(grep -E "^$cat =" $R_AVG|cut -d' ' -f4|sed -e 's/-nan/NA/')
				stddev=$(grep -E "^$cat =" $R_STD|cut -d' ' -f4|sed -e 's/-nan/NA/')
				skew=$(grep -E "^$cat =" $R_SKW|cut -d' ' -f4|sed -e 's/-nan/NA/')
				kurt=$(grep -E "^$cat =" $R_KUR|cut -d' ' -f4|sed -e 's/-nan/NA/')
				echo "$cat|$class|$crop|$count|$min|$max|$avg|$stddev|$skew|$kurt" >>$OUTDIR/NDVI_${d}_${i}_${LC}_stats.txt
			fi
		done
	done
done

rm -f $R_CAT $R_CNT $R_MIN $R_MAX $R_AVG $R_STD $R_SKW $R_KUR

$G_CLEAN
