#!/bin/sh
#
# This scripts extracts zonal statistics for SSM maps in Spain
# using base land cover maps for the area which is available as a single date and pre-loaded.
#

set -e

TAG=p_ssm40m_sensagri_img_th33_mean_run_n4_v1_2_20190315_dsc
LC=spain_fields_4ha_lc
OUTDIR=$SM_TOOLS/streams_crossing/results
YEAR=2017
SSM_BANDS=1

# The SSM maps are 40x40m^2 wide.
#MINCELLS=7
MINCELLS=7

ROWS=$(spatialite -batch -noheader $SM_TOOLS_DB <<EOF
select p.filename,date(p.reference_datetime) from Product as p, Site as s 
where p.tag='$TAG' and strftime('%Y',p.reference_datetime)='$YEAR' 
and s.id=5 and ST_Intersects(p.footprint,s.aoi)>0;
EOF
)

R_CNT=$(mktemp -q)
R_MIN=$(mktemp -q)
R_MAX=$(mktemp -q)
R_AVG=$(mktemp -q)
R_STD=$(mktemp -q)
R_SKW=$(mktemp -q)
R_KUR=$(mktemp -q)

for r in $ROWS
do
	f=$(echo $r|cut -d\| -f1)
	d=$(echo $r|cut -d\| -f2)
	for i in $SSM_BANDS
	do
		echo "SSM $d $i" 
        if [ -f $OUTDIR/SSM_${d}_${i}_${LC}_stats.txt ]
        then
            continue
        fi

		SSM=$SM_TOOLS_REPO/$f
		$G r.in.gdal band=$i in=$SSM out=SSM --o
		$G r.null setnull=0 map=SSM
		$G g.region rast=SSM 
		$G r.resample out=LC in=$LC --o

		$G r.stats.zonal2 -r base=LC cover=SSM rules=$R_CNT method=count out=SSM_count --o
		$G r.stats.zonal2 -r base=LC cover=SSM rules=$R_MIN method=min out=SSM_min --o
		$G r.stats.zonal2 -r base=LC cover=SSM rules=$R_MAX method=max out=SSM_max --o
		$G r.stats.zonal2 -r base=LC cover=SSM rules=$R_AVG method=average out=SSM_mean --o
		$G r.stats.zonal2 -r base=LC cover=SSM rules=$R_STD method=stddev2 out=SSM_stddev --o
		$G r.stats.zonal2 -r base=LC cover=SSM rules=$R_SKW method=skewness2 out=SSM_skewness --o
		$G r.stats.zonal2 -r base=LC cover=SSM rules=$R_KUR method=kurtosis2 out=SSM_kurtosis --o
		echo "map|cat|crop|count|min|max|avg|stddev|skew|kurt" >$OUTDIR/SSM_${d}_${i}_${LC}_stats.txt
		for cat in $($G r.category map=LC) 
		do
			class=$(grep -E "^$cat[[:space:]]" $SM_TOOLS/streams_crossing/spain_fields_4ha_landcover_cat.txt|cut -f2)
			count=$(grep -E "^$cat =" $R_CNT|cut -d' ' -f4|cut -d. -f1)
			# skip when number of cells is too low
			if [ "$count" -ge $MINCELLS ]
			then
				crop=$(grep -E "^$class = " $SM_TOOLS/streams_crossing/spain_fields_reclass_rules.txt|cut -d= -f2|cut -d' ' -f3-)
				min=$(grep -E "^$cat =" $R_MIN|cut -d' ' -f4|sed -e 's/-nan/NA/')
				max=$(grep -E "^$cat =" $R_MAX|cut -d' ' -f4|sed -e 's/-nan/NA/')
				avg=$(grep -E "^$cat =" $R_AVG|cut -d' ' -f4|sed -e 's/-nan/NA/')
				stddev=$(grep -E "^$cat =" $R_STD|cut -d' ' -f4|sed -e 's/-nan/NA/')
				skew=$(grep -E "^$cat =" $R_SKW|cut -d' ' -f4|sed -e 's/-nan/NA/')
				kurt=$(grep -E "^$cat =" $R_KUR|cut -d' ' -f4|sed -e 's/-nan/NA/')
				echo "$cat|$class|$crop|$count|$min|$max|$avg|$stddev|$skew|$kurt" >>$OUTDIR/SSM_${d}_${i}_${LC}_stats.txt
			fi
		done
	done
done

rm -f $R_CNT $R_MIN $R_MAX $R_AVG $R_STD $R_SKW $R_KUR

$G_CLEAN

